@@ -314,9 +314,6 @@ static void icarus_initialise(struct cgpu_info *icarus, int baud)
 	if (icarus->usbinfo.nodev)
 		return;
 
-	usb_set_cps(icarus, baud / 10);
-	usb_enable_cps(icarus);
-
 	interface = _usb_interface(icarus, info->intinfo);
 	ident = usb_ident(icarus);
 
@@ -786,7 +783,7 @@ static void get_options(int this_option_offset, struct cgpu_info *icarus, int *b
 	}
 }
 
-static bool icarus_detect_one(struct libusb_device *dev, struct usb_find_devices *found)
+static struct cgpu_info *icarus_detect_one(struct libusb_device *dev, struct usb_find_devices *found)
 {
 	int this_option_offset = ++option_offset;
 	struct ICARUS_INFO *info;
@@ -1010,7 +1007,7 @@ cmr2_retry:
 		}
 	}
 
-	return true;
+	return icarus;
 
 unshin:
 
@@ -1022,7 +1019,7 @@ shin:
 
 	icarus = usb_free_cgpu(icarus);
 
-	return false;
+	return NULL;
 }
 
 static void icarus_detect(bool __maybe_unused hotplug)
