@@ -21,11 +21,17 @@ bool opt_log_output = false;
 /* per default priorities higher than LOG_NOTICE are logged */
 int opt_log_level = LOG_NOTICE;
 
-static void my_log_curses(int prio, const char *datetime, const char *str)
+static void my_log_curses(int prio, const char *datetime, const char *str, bool force)
 {
 	if (opt_quiet && prio != LOG_ERR)
 		return;
 
+	/* Mutex could be locked by dead thread on shutdown so forcelog will
+	 * invalidate any console lock status. */
+	if (force) {
+		mutex_trylock(&console_lock);
+		mutex_unlock(&console_lock);
+	}
 #ifdef HAVE_CURSES
 	extern bool use_curses;
 	if (use_curses && log_curses_only(prio, datetime, str))
@@ -44,7 +50,7 @@ static void my_log_curses(int prio, const char *datetime, const char *str)
 /*
  * log function
  */
-void _applog(int prio, const char *str)
+void _applog(int prio, const char *str, bool force)
 {
 #ifdef HAVE_SYSLOG_H
 	if (use_syslog) {
@@ -77,6 +83,6 @@ void _applog(int prio, const char *str)
 			fflush(stderr);
 		}
 
-		my_log_curses(prio, datetime, str);
+		my_log_curses(prio, datetime, str, force);
 	}
 }
